from requests import Response
from volworld_common.test.Timer import Timer
from volworld_aws_api_common.api.AA import AA
from volworld_aws_api_common.test.aws.ATestRequest import ATestRequest
from volworld_aws_api_common.test.request import post_request, get_request
from volworld_aws_api_common.test.aws.url import URL_ROOT

from volworld_aws_api_common.test.api.OpenApiValidation import OpenApiValidation


def request_open_api_post_validation(url: str, req, resp_json, resp: Response, attList):
    url = url.replace(URL_ROOT, '')
    url = url.replace('//', '/')

    val = OpenApiValidation()
    val.validate_POST_request(req, url, attList)

    print("[db_fn_open_api_validation] res = ", resp_json)
    print("[db_fn_open_api_validation] resp.status_code = ", resp.status_code)
    if AA.___Error___ not in resp_json:
        val.validate_POST_response({
            AA.Data: resp_json[AA.Data]
        }, url, resp.status_code, attList)
    else:
        val.validate_POST_response({
            AA.___Error___: resp_json[AA.___Error___]
        }, url, resp.status_code, attList)


def request_open_api_get_validation(url: str, resp_json, resp: Response, attList):
    url = url.replace(URL_ROOT, '')
    url = url.replace('//', '/')

    val = OpenApiValidation()

    print("[db_fn_open_api_validation] res = ", resp_json)
    print("[db_fn_open_api_validation] resp.status_code = ", resp.status_code)
    if AA.___Error___ not in resp_json:
        val.validate_GET_response({
            AA.Data: resp_json[AA.Data]
        }, url, resp.status_code, attList)
    else:
        val.validate_GET_response({
            AA.___Error___: resp_json[AA.___Error___]
        }, url, resp.status_code, attList)

def post_request_with_validation(
        url: str, req: dict,
        test_req: ATestRequest, attList,
        token: str = None):
    with Timer(f"Call DB Function [{url}]"):
        resp_json, resp = post_request(url, req, test_req, attList, token=token)
    request_open_api_post_validation(url, req, resp_json, resp, attList)
    return resp_json, resp

def get_request_with_validation(
        url: str,
        test_req: ATestRequest, attList,
        token: str = None):
    with Timer(f"Call DB Function [{url}]"):
        resp_json, resp = get_request(url, test_req, attList, token=token)
    request_open_api_get_validation(url, resp_json, resp, attList)
    return resp_json, resp



from volworld_aws_api_common.api.enum.HttpStatus import HttpStatus
from volworld_common.util.id_util import new_rand_test_user_name
from volworld_aws_api_common.api.AA import AA, AAList
from volworld_aws_api_common.api.Aws import Aws
from volworld_aws_api_common.api.url import authUrl
from volworld_aws_api_common.test.aws.ATestRequest import ATestRequest
from volworld_aws_api_common.test.request import post_request
from volworld_aws_api_common.test.aws.request.request_util import response_to_dict


def post__signup(
        name: str, pw: str,
        req: ATestRequest,
        ):
    resp_json, resp = post_request(
        authUrl.doSignupUrl, {
            AA.Name: name,
            AA.Password: pw,
        }, req, AAList
        )
    return response_to_dict(resp_json, resp)


def act__signup(name: str = None, pw: str = None) -> dict:
    if name is None:
        name = new_rand_test_user_name()
    if pw is None:
        pw = 'password'
    resp = post__signup(name, pw, ATestRequest(True))
    print(f"[act__signup] resp = ", resp)
    if resp[AA.Data] is not None:
        assert resp[AA.HttpStatus] == HttpStatus.Created_201.value
        assert resp[AA.Data][AA.Ok] is True
        resp[AA.Name] = name
        resp[AA.Password] = pw
        return resp
    else:
        assert AA.___Error___ in resp
    print(f"[act__signup] return resp = ", resp)
    return resp



from volworld_aws_api_common.api.AA import AA
from volworld_aws_api_common.api.url import authUrl
from volworld_aws_api_common.test.aws.ATestRequest import ATestRequest
from volworld_aws_api_common.test.request import get_request
from volworld_aws_api_common.test.aws.request.request_util import response_to_dict
from volworld_aws_api_common.api.enum.HttpStatus import HttpStatus

def get__current_user(
        req: ATestRequest,
        token: str = None):
    resp_json, resp = get_request(
        authUrl.currentUserUrl, req,
        token=token)
    return response_to_dict(resp_json, resp)

def act__current_user(token: str = None) -> dict:
    resp = get__current_user(ATestRequest(True), token)
    assert resp[AA.HttpStatus] == HttpStatus.Ok_200
    return resp

def act__current_user___error(token: str = None) -> dict:
    resp = get__current_user(ATestRequest(True), token)
    assert AA.___Error___ in resp
    return resp
